/*#######################################################################################
AVR Small Webserver

Copyright (C) 2004 Ulrich Radig

#######################################################################################*/

#ifndef _TCP_H
#define _TCP_H

#include <avr/pgmspace.h>

struct TCP_Socket
{
    unsigned int Store;		///< source port used also as socket id
    PGM_P Pointer;
    unsigned char Status;

#if USE_MMC
    unsigned int Cluster;
    unsigned long File_Size;
    unsigned int File_Block_Counter;
#endif
};

//Prototype
extern struct TCP_Socket tcpsockets[];
extern struct TCP_Socket *tcp_socket;

extern PGM_P pointerold;

extern void tcp_store (char *);
extern unsigned char tcp_add_data (PGM_P ,char *,int *);
extern char tcp_add_datastring (char *pointer,char *buffer,int *bufferlen);
extern void TCP_New_Packtlen (char *buffer,int *bufferlen,unsigned int result16);
extern void tcp_make (char *);
extern void TCP_PORT_CLOSE (char *,int *);
extern int TCP_OPEN (char *,int *);
extern char TCP_Stack (char *buffer,int *bufferlen,unsigned char *Stackpointer);
extern void tcp_make_client (char *buffer);

#define Max_TCP_Sockets  7

#define TCP_OFFSET				0x22	///< TCP header offset

/*
TCP Checksum: 16 bits 

    The checksum field is the 16 bit one's complement of the one's
    complement sum of all 16 bit words in the header and text.  If a
    segment contains an odd number of header and text octets to be
    checksummed, the last octet is padded on the right with zeros to
    form a 16 bit word for checksum purposes.  The pad is not
    transmitted as part of the segment.  While computing the checksum,
    the checksum field itself is replaced with zeros.

    The checksum also covers a 96 bit pseudo header conceptually
    prefixed to the TCP header.  This pseudo header contains the Source
    Address, the Destination Address, the Protocol, and TCP length.
    This gives the TCP protection against misrouted segments.  This
    information is carried in the Internet Protocol and is transferred
    across the TCP/Network interface in the arguments or results of
    calls by the TCP on the IP.

                     +--------+--------+--------+--------+
                     |           Source Address          |
                     +--------+--------+--------+--------+
                     |         Destination Address       |
                     +--------+--------+--------+--------+
                     |  zero  |  PTCL  |    TCP Length   |
                     +--------+--------+--------+--------+

      The TCP Length is the TCP header length plus the data length in
      octets (this is not an explicitly transmitted quantity, but is
      computed), and it does not count the 12 octets of the pseudo
      header.
*/

//TCP Header Layout (size = 20B)
struct TCP_Header
{
    unsigned int 	TCP_SrcPort;	//der Quellport fr die Verbindung (Socket)
    unsigned int 	TCP_DestPort;	//der Zielport fr die Verbindung (Socket)
    unsigned char	TCP_Seqnum[4];	//numerierter Bytestrom
    unsigned char	TCP_Acknum[4];	//numerierter Bytestrom (Gegenrichtung)
    unsigned char 	TCP_HdrFlags[2];//4Bit Lnge des Headers in 32 Bit schritten rest Flags
    unsigned int	TCP_Window;		//Fenstergre max. 64K Byte
    unsigned int	TCP_Chksum;		//Enthlt eine Prfsumme ber Header und Datenbereich
    unsigned int	TCP_UrgentPtr;	//16 Bit fr dringende bertragung
};

#define	TCP_SRCPORT				TCP_OFFSET+0x00	//der Quellport fr die Verbindung (Socket)
#define	TCP_DESTPORT			TCP_OFFSET+0x02	//der Zielport fr die Verbindung (Socket)
#define	TCP_SEQNUM				TCP_OFFSET+0x04	//numerierter Bytestrom
#define	TCP_ACKNUM				TCP_OFFSET+0x08	//numerierter Bytestrom (Gegenrichtung)
#define	TCP_HDRFLAGS			TCP_OFFSET+0x0C	//4Bit Lnge des Headers in 32 Bit schritten
//rest Flags
#define	TCP_WINDOW				TCP_OFFSET+0x0E	//Fenstergre max. 64K Byte
#define	TCP_CHKSUM				TCP_OFFSET+0x10	//Enthlt eine Prfsumme ber Header und
//Datenbereich
#define	TCP_URGENTPTR			TCP_OFFSET+0x12	//16 Bit fr dringende bertragung

//TCP Flags
#define  FIN_FLAG				0x01	//Datenbertragung beendet (finished)
#define  SYN_FLAG				0x02	///< set seq number (synchronize) flag
#define  RST_FLAG				0x04	//Verbindung neu initialisieren (reset)
#define  PSH_FLAG				0x08	//Datenbergabe an Anwendung beschleunigen (push)
#define  ACK_FLAG				0x10	//Datenbertragung besttigen (acknowledged)

//TCP_Status
#define TCP_SOCKET_NOT_USE		0	//TCP Port geschlossen
#define TCP_SOCKET_OPEN1		1	//TCP Port open
#define TCP_SOCKET_OPEN2		2	//TCP Port open 1. ACK wurde abgearbeitet
#define TCP_SOCKET_CLOSE		3	//TCP Port wird bei nchsten ACK geschlossen


#endif // _TCP_H
